package
{
	//import flash.desktop.NativeApplication;
	import flash.display.Bitmap;
	import flash.events.Event;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.text.Font;
	import flash.ui.Multitouch;
	import flash.ui.MultitouchInputMode;
	import com.greensock.plugins.TweenPlugin;
	import com.greensock.plugins.HexColorsPlugin;
	import map.Destination;
	import map.MapData;
	import map.MapLayer;
	import ui.button.FlatButton;
	import ui.button.LinkedPanelButton;
	import ui.button.SideButton;
	import ui.button.InfoButton;
	import ui.button.SideButtonLayers;
	import ui.LayersPanel;
	import ui.panel.*;
	import ui.RouteHUD;
	import ui.Spritesheet;
	import ui.StateEvent;
	import ui.Toast;
	import ui.ToastEvent;
	import ui.RouteEvent;
	import ui.RouteSlot;
	import com.greensock.TweenLite;
	
	/**
	 * Main
	 * @author Cassandra de Git and Charlie Chao
	 */
	public class Main extends Sprite 
	{
		
		/**
		 * Constants
		 */
		public static const RESOURCES_TOTAL:int = 3;			// total external resources
		public static const PATH_UI:String = "ui.json";					
		public static const PATH_IMAGES:String = "images.json";				
		public static const PATH_MAP_DATA:String = "mapData.json";				
		// categories
		public static const CATEGORY_MAIN_MENU:int = 0;
		public static const CATEGORY_QUICK_FIND:int = 1;
		public static const CATEGORY_ROUTE_PLANNER:int = 2;
		// panel ids
		public static const PANEL_NONE:int = -1;				// no panel
		public static const PANEL_MAIN_MENU:int = 0;			// main menu (access Activities, Facilities, Events, About)
		public static const PANEL_ACTIVITIES:int = 1;			// activities
		public static const PANEL_FACILITIES:int = 2;			// facilities (Points of Interest)
		public static const PANEL_QUICK_FIND:int = 3;			// quick find
		public static const PANEL_ROUTE_PLANNER:int = 4;		// route planner
		public static const PANEL_INFO:int = 5;					// information (for locations)
		public static const PANEL_EVENTS:int = 6;				// events
		public static const PANEL_ABOUT:int = 7;				// about
		
		/**
		 * Attributes
		 */
		// resources
		//[Embed(source='assets/DINEngschriftStd.otf', fontFamily='DIN Engschrift Std', mimeType='application/x-font')] 
		//public var fontDIN:Class;
		[Embed(source='assets/AkzidenzGrotesk-BoldCondAlt.otf', fontFamily='AkzBold', mimeType='application/x-font', embedAsCFF="false")]
		public var fontAkzBold:Class;
		[Embed(source='assets/AkzidenzGrotesk-LightCond.otf', fontFamily='AkzLight', mimeType='application/x-font', embedAsCFF="false")]
		public var fontAkzLight:Class;
		public var uiSpriteSheet:Spritesheet;		// UI elements sprite sheet
		public var imageSpriteSheet:Spritesheet;	// images sprite sheet (300x160) or (260x160)
		public var mapData:MapData;					// activities and locations data
		public var resourcesLoaded:int;
		public var routeList:Vector.<Destination>;
		public static var selectedDestination:int;
		
		/**
		 * Components
		 */
		public var mapLayer:MapLayer;					// map layer
		public var activePanelID:int;					// currently visible panel
		public var panels:Vector.<Panel>;				// all panels
		public var sideButtons:Vector.<SideButton>;		// side buttons
		public var layersButton:FlatButton;				// layers button
		public var layersPanel:LayersPanel;				// layers panel (handles layer toggle)
		public var toast:Toast;
		public var routeHUD:RouteHUD;	
		
		/**
		 * Main
		 */
		public function Main():void 
		{
			//Font.registerFont(fontAkzBold);
			//Font.registerFont(fontAkzLight);
			// setup stage
			stage.scaleMode = flash.display.StageScaleMode.NO_SCALE;
			stage.align = flash.display.StageAlign.TOP_LEFT;			
			stage.addEventListener(Event.DEACTIVATE, deactivate);
			TweenPlugin.activate([HexColorsPlugin]); // allows hex color tweening for the TweenLite plugin
			Multitouch.inputMode = MultitouchInputMode.GESTURE;
			
			// setup models
			routeList = new Vector.<Destination>;
			activePanelID = PANEL_NONE;
			selectedDestination = -1;

			// load spritesheet
			uiSpriteSheet = new Spritesheet(PATH_UI);
			uiSpriteSheet.addEventListener(StateEvent.READY, initializeUI);
			imageSpriteSheet = new Spritesheet(PATH_IMAGES);
			imageSpriteSheet.addEventListener(StateEvent.READY, initializeUI);
			mapData = new MapData(PATH_MAP_DATA);
			mapData.addEventListener(StateEvent.READY, initializeUI);
		}
		
		/**
		 * Initializes all UI elements.
		 */
		private function initializeUI(e:StateEvent):void 
		{
			resourcesLoaded++; // increases resource count
			if (resourcesLoaded == RESOURCES_TOTAL) { // all resources loaded...
				// remove event listener
				uiSpriteSheet.removeEventListener(StateEvent.READY, initializeUI);
				imageSpriteSheet.removeEventListener(StateEvent.READY, initializeUI);
				mapData.removeEventListener(StateEvent.READY, initializeUI);
				
				// sets panel resources reference
				Panel.uiSpriteSheet = uiSpriteSheet;
				Panel.imageSpriteSheet = imageSpriteSheet;
				MapLayer.uiSpriteSheet = uiSpriteSheet;
				RouteSlot.closeIcon = uiSpriteSheet.getSprite("panel_close_small");
				RouteSlot.upIcon = uiSpriteSheet.getSprite("panel_up");
				RouteSlot.downIcon = uiSpriteSheet.getSprite("panel_down");
				
				// initialize UI components
					
					// map
					mapLayer = new MapLayer(mapData, routeList);
					
					// panels
					panels = new Vector.<Panel>;
					panels.push(new MainMenuPanel());
					panels.push(new ActivitiesPanel(mapData));
					panels.push(new FacilitiesPanel(mapData));
					panels.push(new QuickFindPanel());
					panels.push(new RoutePlannerPanel(routeList));
					panels.push(new InfoPanel(mapData));
					panels.push(new EventsPanel());
					panels.push(new AboutPanel());
					
					// construct buttons with the panels initialized
					for each (var p:Panel in panels) { p.constructButtons(panels); }
					
					// side buttons
					sideButtons = new Vector.<SideButton>;
					sideButtons.push(new SideButton(0, 0, SideButton.BUTTON_SIZE, SideButton.BUTTON_SIZE, uiSpriteSheet.getSprite("side_button_main_menu"), PANEL_MAIN_MENU));
					sideButtons.push(new SideButton(0, 0, SideButton.BUTTON_SIZE, SideButton.BUTTON_SIZE, uiSpriteSheet.getSprite("side_button_quick_find"), PANEL_QUICK_FIND));
					sideButtons.push(new SideButton(0, 0, SideButton.BUTTON_SIZE, SideButton.BUTTON_SIZE, uiSpriteSheet.getSprite("side_button_route_planner"), PANEL_ROUTE_PLANNER));
					
					// layers module
					layersButton = new SideButtonLayers(0, SideButton.BUTTON_SIZE, SideButton.BUTTON_SIZE, SideButton.BUTTON_SIZE, uiSpriteSheet.getSprite("side_button_layers"));
					layersButton.bgColor = CommonStyle.SIDE_BUTTON_COLOR_BG;
					layersButton.bgColorDown = CommonStyle.SIDE_BUTTON_COLOR_BG_DOWN;
					layersButton.listen();
					layersButton.addEventListener(MouseEvent.CLICK, toggleLayersPanel);
					layersPanel = new LayersPanel(uiSpriteSheet);
					layersPanel.y = layersButton.y;
					
					// toast
					toast = new Toast();
					
					// route HUD
					routeHUD = new RouteHUD();
					
					// add components to stage
					addChild(mapLayer);
					addChild(sideButtons[CATEGORY_MAIN_MENU]);
					addChild(sideButtons[CATEGORY_QUICK_FIND]);
					addChild(sideButtons[CATEGORY_ROUTE_PLANNER]);
					addChild(layersPanel);
					addChild(layersButton);
					addChild(routeHUD);
					addChild(toast);
				
				// position UI elements based on screen size
				stage.addEventListener(Event.RESIZE, resizeUI);
				stage.addEventListener(PanelEvent.PANEL_OPEN, openPanel);
				stage.addEventListener(ToastEvent.SHOW, showToast);
				stage.addEventListener(RouteEvent.CHANGE, routeChange);
				sideButtons[PANEL_MAIN_MENU].dispatchEvent(new MouseEvent(MouseEvent.CLICK));
				
				resizeUI();
			}
		}
		
		/**
		 * Re-adjust UI elements position.
		 */
		private function resizeUI(e:Event = null):void 
		{
			// side buttons
			sideButtons[CATEGORY_MAIN_MENU].x = sideButtons[CATEGORY_QUICK_FIND].x = sideButtons[CATEGORY_ROUTE_PLANNER].x = stage.stageWidth - SideButton.BUTTON_SIZE;
			sideButtons[CATEGORY_MAIN_MENU].y = sideButtons[CATEGORY_QUICK_FIND].y = sideButtons[CATEGORY_ROUTE_PLANNER].y = stage.stageHeight / 2 - SideButton.BUTTON_SIZE / 2;
			sideButtons[CATEGORY_MAIN_MENU].y -=  SideButton.BUTTON_SIZE + SideButton.BUTTON_SPACING;
			sideButtons[CATEGORY_ROUTE_PLANNER].y += SideButton.BUTTON_SIZE + SideButton.BUTTON_SPACING;
			
			// layers modules
			layersButton.x = stage.stageWidth - SideButton.BUTTON_SIZE;
			layersPanel.x = layersButton.x - LayersPanel.WIDTH - 15;
			
			// active panel
			if (activePanelID != PANEL_NONE) panels[activePanelID].render();
			mapLayer.render();
		}
		
		/**
		 * Handles panel open event.
		 */
		private function openPanel(e:PanelEvent = null):void 
		{
			var panelID:int = e.params.panel;
			var linkedPanel:Panel = (e.params != null && e.params.panel != PANEL_NONE)?panels[e.params.panel]:null;
			var activePanel:Panel = (activePanelID != PANEL_NONE)?panels[activePanelID]:null;
			
			// set side buttons state
			for (var i:int = 0; i < sideButtons.length; i++) {
				if (linkedPanel == null || i != linkedPanel.category) 
					sideButtons[i].setState(false);
			}
			
			// animate panel
			if (activePanelID == PANEL_INFO && linkedPanel == activePanel) {
				// update info panel instead of reloading it when it is already opened
				InfoPanel(panels[PANEL_INFO]).loadInfo(e.params.id);
			} else if (linkedPanel != null && activePanel != linkedPanel) { // panel is not visible...
				// show panel
				// FIXME: infinite back button loop when going in circle
				// set ids
				var previousPanelID:int = activePanelID;
				activePanelID = panelID;
				
				// fade in new panel
				if (this.contains(linkedPanel)) linkedPanel.fadeIn();	// fades panel in if it is still on screen
				else addChild(linkedPanel); // adds panel to screen
				
				// info panel load data
				if (linkedPanel is InfoPanel && e.params.id != null) {
					InfoPanel(panels[PANEL_INFO]).loadInfo(e.params.id);
				}
				
				// deal with previous panel
				if (activePanel != null) {
					// set previous panel for back button functionality
					if (activePanel.previousPanel != panelID)
						panels[panelID].previousPanel = previousPanelID;
					// fade out previous panel
					activePanel.fadeOut();
				}
				
				// re-prioritize toast
				setChildIndex(toast, numChildren - 1);
				
				// move route HUD
				TweenLite.killTweensOf(routeHUD);
				TweenLite.to(routeHUD, Panel.FADE_DURATION, { x: linkedPanel.panelWidth + RouteHUD.EDGE_OFFSET } );
			} else {
				// close the panel
				if (activePanel != null) {
					activePanel.fadeOut(); // fades out panel
					// move route HUD
					TweenLite.killTweensOf(routeHUD);
					TweenLite.to(routeHUD, Panel.FADE_DURATION, { x: RouteHUD.EDGE_OFFSET } );
				}
				activePanelID = PANEL_NONE; // remove reference
				for each (var p:Panel in panels) { p.previousPanel = PANEL_NONE; } // clears all previous panel reference from all panels
				if (panelID != PANEL_NONE) sideButtons[panels[panelID].category].setState(false); // deactivate side button
				
			}
		}
		
		/**
		 * Show toast notification.
		 */
		public function showToast(e:ToastEvent):void 
		{
			toast.showToast(e.title, e.message);
		}
		
		/**
		 * Toggles visibility of layers panel
		 */
		private function toggleLayersPanel(e:MouseEvent = null):void
		{
			if (!layersPanel.visible) layersPanel.fadeIn();
			else layersPanel.fadeOut();
		}
		
		/**
		 * Handles route list change.
		 */
		private function routeChange(e:RouteEvent):void 
		{
			if (e.params != null) {
				var data:Object = mapData.getLocation(e.params.id);
				switch (e.params.action) {
					case "remove":
						routeList.splice(e.params.id, 1);
						break;
					case "swap":
						var temp:Destination = routeList[e.params.id2];
						routeList[e.params.id2] = routeList[e.params.id];
						routeList[e.params.id] = temp;
						break;
					case "insertBefore":
						routeList.unshift(new Destination(e.params.id, data.name, 0, data.icon));
						break;
					case "insertAfter":
						routeList.push(new Destination(e.params.id, data.name, 0, data.icon));
						break;
					case "insertIntersection":
						var tempList:Vector.<Destination> = routeList.slice(selectedDestination, routeList.length);
						routeList.splice(selectedDestination, routeList.length);
						routeList.push(new Destination(e.params.id, "Intersection", 0, "poi_intersection"));
						for each (var o:Destination in tempList) {
							routeList.push(o);
						} 
						break;
				}
			} 
			
			// update view
			if (activePanelID == PANEL_ROUTE_PLANNER) {
				mapLayer.showRoute(true);
				panels[PANEL_ROUTE_PLANNER].update();
			} else {
				mapLayer.showRoute();
			}
			if (routeList.length > 0) routeHUD.update(routeList[0], uiSpriteSheet.getSprite(routeList[0].icon + "_small"));
			else routeHUD.update();
		}
		
		/**
		 * Closes app.
		 */
		private function deactivate(e:Event):void 
		{
			// auto-close
			//NativeApplication.nativeApplication.exit();
		}
		
	}
	
}