package map
{

	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.StageQuality;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.TransformGestureEvent;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Vector3D;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.text.TextField;
	import ui.RouteEvent;
	import ui.panel.PanelEvent;
	import ui.Spritesheet;
	import ui.ToastEvent;
	import com.greensock.TweenLite;
	import com.greensock.easing.Sine;
	import assets.Map;
	import assets.MapLabels;
	
	/**
	 * MapLayer
	 * Shows the map layer.
	 * @author Charlie Chao
	 */
	public class MapLayer extends Sprite 
	{
		/**
		 * Constants
		 */
		public static var PATH_NODES:String = "nodes7.nl";;
		public static var ZOOM_MIN:Number = 0.6;
		public static var ZOOM_MAX:Number = 4;
		public static var ZOOM_RATE:Number = 0.2;
		public static var DESTINATION_THRESHOLD:Number = 20;
		public static var PAN_DURATION:Number = 0.75;
		
		/**
		 * Statics
		 */
		public static var uiSpriteSheet:Spritesheet;		// sprite sheet reference
		
		/**
		 * Attributes
		 */
		private var parkMap:Map;
		private var facilitiesLayer:Sprite;
		private var labelsLayer:Sprite;
		private var markersLayer:Sprite;
		//private var map:LabelLayer;
		//private var map:Map;
		//private var map:Map;
		
		private var dragging:Boolean;
		//public var cx:int, cy:int;			// current position
		private var px:int, py:int;			// previous position
		private var zoom:Number;
		// person
		private var personMarker:PersonMarker;
		private var routeList:Vector.<Destination>;
		private var routeMarkers:Vector.<RouteMarker>;
		
		/**
		 * Node Graph
		 */
		public var graph:Graph = new Graph();
		public var nodes:Array = new Array();
		public var edges:Array = new Array();
		private var nodesAll:Vector.<Node> = new Vector.<Node>;
		private var nodesLocation:Vector.<Location> = new Vector.<Location>;
		private var nodesWashroom:Vector.<Node> = new Vector.<Node>;
		private var nodesParking:Vector.<Node> = new Vector.<Node>;
		private var nodesPicnic:Vector.<Node> = new Vector.<Node>;
		private var nodesTrash:Vector.<Node> = new Vector.<Node>;
		private var nodesIntersection:Vector.<Node> = new Vector.<Node>;
		//private var currentNodes:Array = new Array();
		private var astar:Astar;
		private var edge:Edge;
		// data loading
		private var nodeLoader:URLLoader;
		private var mapData:MapData;
		//private var locationsLoader:URLLoader = new URLLoader();
		//private var label:TextField;
		//private var data:XML;
		//private var xmlLoaded:Boolean = false;
		//private var textParser:TextParser;
		//private var routePlanner:RoutePlanner = new RoutePlanner();
		
		public function MapLayer(mapData:MapData, routeList:Vector.<Destination>) 
		{
			this.mapData = mapData;
			this.routeList = routeList;
			zoom = 1;
			routeMarkers = new Vector.<RouteMarker>;
			
			// setup maps
			parkMap = new Map();
			//parkMap.cacheAsBitmapMatrix = parkMap.transform.concatenatedMatrix;
			facilitiesLayer = new Sprite();
			//facilitiesLayer.cacheAsBitmapMatrix = facilitiesLayer.transform.concatenatedMatrix;
			markersLayer = new Sprite();
			//markersLayer.cacheAsBitmapMatrix = markersLayer.transform.concatenatedMatrix;
			labelsLayer = new Sprite();
			labelsLayer.addChild(new MapLabels);
			
			// setup cache
			//cacheAsBitmap = facilitiesLayer.cacheAsBitmap = parkMap.cacheAsBitmap = markersLayer.cacheAsBitmap = true;
			cache();
			
			// load node data
			//loadLocations();
			nodeLoader = new URLLoader();
			nodeLoader.addEventListener(Event.COMPLETE, processNodes);
			nodeLoader.load(new URLRequest(PATH_NODES));
			
			// add components to screen
			parkMap.addChild(labelsLayer);
			parkMap.addChild(facilitiesLayer);
			parkMap.addChild(markersLayer);
			addChild(parkMap);
			
			// add to stage listener
			addEventListener(Event.ADDED_TO_STAGE, added);
		}
		
		// added to screen handler
		public function added(e:Event = null):void 
		{
			removeEventListener(Event.ADDED_TO_STAGE, added);
			
			// add node listener
			//addEventListener(NodeClicked.NODE_CLICKED, nodeClickedHandler);
			
			// attach listeners for direct manipulation
			parent.addEventListener(MouseEvent.MOUSE_UP, mouseUp);
			parent.addEventListener(MouseEvent.MOUSE_MOVE, mouseMove);
			//addEventListener(MouseEvent.RIGHT_CLICK, positionPerson);
			addEventListener(MouseEvent.CLICK, setIntersection);
			addEventListener(MouseEvent.MOUSE_DOWN, mouseDown);
			addEventListener(MouseEvent.MOUSE_WHEEL, mouseWheel);
			addEventListener(TransformGestureEvent.GESTURE_ZOOM, onZoom);
			
			// attach listeners for change
			parent.addEventListener(MapEvent.CHANGE, changeHandler);
			parent.addEventListener(MapEvent.LAYER_TOGGLE, toggleHandler);
		}
		
		// renders map layer
		public function render():void 
		{
			// draw base
			graphics.clear();
			graphics.beginFill(0x222222);
			graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			graphics.endFill();
		}
		
		// zooms map elements to specified position
		private function zoomMap(originX:int, originY:int, zoomChange:Number):void
		{
			// FIXME: change zoom
			zoom += zoomChange;
			if (zoom < ZOOM_MIN) zoom = ZOOM_MIN;
			else if (zoom > ZOOM_MAX) zoom = ZOOM_MAX;
			
			// change position
			var cx:int = originX;// + parkMap.x;
			var cy:int = originY;// + parkMap.y;
			
			parkMap.scaleX = parkMap.scaleY = zoom;
			
			// apply matrix
			var m:Matrix = parkMap.transform.matrix;
			m.identity();
			m.transformPoint(new Point(originX, originY));
			m.translate(-cx, -cy);
			m.scale(zoom, zoom);
			m.translate(cx, cy);
			m.transformPoint(new Point());
			parkMap.transform.matrix = m;
		}
		
		// mouse down handler
		private function mouseDown(e:MouseEvent):void
		{
			dragging = true;
			px = stage.mouseX;
			py = stage.mouseY;
			stage.quality = StageQuality.LOW;
			parkMap.cacheAsBitmap = false;
		}
		
		// mouse move handler
		private function mouseMove(e:MouseEvent):void
		{
			// handle map dragging
			if (dragging) {
				var dx:int = (stage.mouseX - px);
				var dy:int = (stage.mouseY - py);
				parkMap.x += dx;
				parkMap.y += dy;
				px = stage.mouseX;
				py = stage.mouseY;
			}
		}
		
		// mouse up handler
		private function mouseUp(e:MouseEvent):void
		{
			dragging = false;	// stops dragging the map
			stage.quality = StageQuality.HIGH;
			parkMap.cacheAsBitmap = true;
		}
		
		// mouse wheel handler
		public function mouseWheel(e:MouseEvent):void
		{
			// handle map zoom
			if (e.delta > 0) zoomMap(mouseX, mouseY, 0.2);
			else zoomMap(mouseX, mouseY, -0.2);
		}
		
		// gesture zoom handler
		public function onZoom(e:TransformGestureEvent):void
		{
			//scaleLevel += tempScale;
			zoomMap(e.localX, e.localY, (e.scaleX + e.scaleY) / 2);
			//label.text = String(zoom);
		}
		
		// positions person
		public function positionPerson(e:MouseEvent = null):void 
		{
			personMarker.setNode(nearestNode(parkMap.mouseX, parkMap.mouseY, nodesAll));
			// check if person has reached their first destination
			if (routeList.length > 0) {
				var firstNode:Node = findLocation(routeList[0].id);
				if (Math.sqrt(Math.pow(parkMap.mouseX - firstNode.x, 2) + Math.pow(parkMap.mouseY - firstNode.y, 2)) <= DESTINATION_THRESHOLD) {
					dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Destination Reached", "You have reached: " + routeList[0].name + "."));
					dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "remove", id: 0 }));
				}
			}
			dispatchEvent(new RouteEvent(RouteEvent.CHANGE, null));
		}
		
		// sets intersection node if viable
		public function setIntersection(e:MouseEvent = null):void 
		{
			if (Main.selectedDestination != -1) {
				dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "insertIntersection", id: Location(nearestNode(parkMap.mouseX, parkMap.mouseY, nodesIntersection)).id } ));
			}
		}
		
		// returns the nearest node
		public function nearestNode(tx:int, ty:int, list:Vector.<Node>):Node 
		{
			var dist:Number;
			var lowestDist:Number = Number.MAX_VALUE;
			var nodeNearest:Node = null;
			for each (var n:Node in list) {
				dist = Math.sqrt(Math.pow(tx - n.x, 2) + Math.pow(ty - n.y, 2));
				if (dist < lowestDist) {
					lowestDist = dist;
					nodeNearest = n;
				}
			}
			return nodeNearest;
		}
		
		/**
		 * Shows update route from route list.
		 */
		public function showRoute(showAll:Boolean = false):void 
		{
			var n1:Node, n2:Node;
			var i:int, j:int;
			
			// remove route markers
			for (i = 0; i < routeMarkers.length; i++) markersLayer.removeChild(routeMarkers[i]);
			routeMarkers.length = 0;
			
			// find new path
			graph.redraw();
			if (routeList.length != 0) { 
				n1 = findLocation(routeList[0].id);
				astar = new Astar(graph, personMarker.node.getIndex(), n1.getIndex());
				routeList[0].distance = astar.getPath(true);
				j = routeMarkers.push(new RouteMarker(0, n1.x, n1.y, uiSpriteSheet.getSprite("marker"))) - 1;
				markersLayer.addChild(routeMarkers[j]);
				for (i = 1; i < ((showAll)?routeList.length:1); i++) { //trace(routeList.length+"_"+i +"_"+findLocation(routeList[i].id).getIndex()+"_"+findLocation(routeList[i-1].id).getIndex());
					n1 = findLocation(routeList[i - 1].id);
					n2 = findLocation(routeList[i].id);
					astar = new Astar(graph, n1.getIndex(), n2.getIndex());
					routeList[i].distance = astar.getPath(showAll); //trace(routeList[i].distance);
					j = routeMarkers.push(new RouteMarker(i, n2.x, n2.y, uiSpriteSheet.getSprite("marker"))) - 1;
					markersLayer.addChild(routeMarkers[j]);
				}
			}
			// toggle facilities layer
			if (Main.selectedDestination != -1) facilitiesLayer.mouseEnabled = false
			else facilitiesLayer.mouseEnabled = true;
		}
			
		// map change handler
		public function changeHandler(e:MapEvent = null):void 
		{
			// handles panel triggers
			// TODO: move to location
			var targetNode:Node = null;
			if (e.params != null && e.params is String) {
				switch (e.params) {
					case "qf_washroom":
						findNearest(nodesWashroom);
						dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Quick Find", "The nearest washroom has been set as your current destination."));
						break;
					case "qf_parking":
						findNearest(nodesParking);
						dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Quick Find", "The nearest parking lot has been set as your current destination."));
						break;
					case "qf_picnic":
						findNearest(nodesPicnic);
						dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Quick Find", "The nearest picnic area has been set as your current destination."));
						break;
					case "qf_trash":
						findNearest(nodesWashroom as Vector.<Node>);
						dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Quick Find", "The nearest trash bin has been set as your current destination."));
						break;
					default:
						trace(e.params);
						targetNode = findLocation(e.params as String);
						if (targetNode != null) TweenLite.to(parkMap, PAN_DURATION, { x: -targetNode.x + stage.stageWidth / 2, y: -targetNode.y + stage.stageHeight / 2, ease: Sine.easeInOut } );
						break;
				}
			}
		}
		
		// finds the nearest item from node list and adds it to queue
		public function findNearest(list:Vector.<Node>):void 
		{
			var targetNode:Location = nearestNode(personMarker.x, personMarker.y, list) as Location;
			TweenLite.to(parkMap, PAN_DURATION, { x: -targetNode.x + stage.stageWidth / 2, y: -targetNode.y + stage.stageHeight / 2, ease: Sine.easeInOut, onComplete: cache } );
			dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "insertBefore", id: targetNode.id } ));
			uncache();
		}
		
		// disables cache
		public function uncache():void 
		{
			parkMap.cacheAsBitmap = false;
		}
		
		// enables cache
		public function cache():void 
		{
			parkMap.cacheAsBitmap = true;
		}
		
		// map layers toggle handler
		public function toggleHandler(e:MapEvent = null):void 
		{
			// handles panel triggers
			if (e.params != null) {
				var p:Object = e.params;
				switch (p.name) {
					case "Trails":
						trace(p.name + ":" + p.state);
						break;
					case "Labels":
						trace(p.name + ":" + p.state);
						labelsLayer.visible = p.state;
						break;
					case "Facilities":
						trace(p.name + ":" + p.state);
						facilitiesLayer.visible = p.state;
						break;
					case "Tree Density":
						trace(p.name + ":" + p.state);
						break;
					default:
						trace(p);
						break;
				}
			}
		}
		
		/**
		 * Node System Code
		 */
		
		// processes node data
		public function processNodes(event:Event):void
		{
			nodeLoader.removeEventListener(Event.COMPLETE, processNodes);
			// process it here
			var data:Array = event.target.data.split('\n');
			var i:int, j:int;
			var d:Array;
			//nodes.length = 0; // resets current nodes
			//var xx:int;
			for (i = 0; i < data.length; i++) {
				//trace(String(data[i]).replace("i,", "i_"+xx+","));
				//if (String(data[i]).indexOf("i,") >= 0) xx++;
				// add node to model
				d = (String)(data[i]).split(",");
				//nodes.push(new Node(d[1], d[2], d[0]));
				// d[0] is name of node
				//trace(d[1] + ", " + d[2]);
				nodes.push([d[1]-45, d[2]-45, d[0]]);
			}
			
			for (i = 0; i < nodes.length; i++) {
				// add connected nodes to each node in the model
				d = (String)(data[i]).split(",");
				for (j = 3; j < d.length; j++) 
				{
					//nodes[i].addNode(nodes[d[j]]);
					edges.push([i, d[j]]);
				}
			}
			//dispatchEvent(new NodeEvent(NodeEvent.NODE_UPDATE));
			
			createMap();
		}
		
		// creates the map
		public function createMap():void 
		{
			//A loop in order to create all the nodes
            for(var a:int=0;a<nodes.length;a++)
            {
				var node:Node;
				if (nodes[a][2].length > 0) {
					//make it a location instead
					var desc:String = "";
					var data:Object = mapData.getLocation(nodes[a][2]);
					var icon:BitmapData = (data != null)?uiSpriteSheet.getSprite(data.icon + "_small"):null; // TODO: marker icon
					node = new Location(Graph.getNextIndex(), new Vector3D(nodes[a][0], nodes[a][1]), nodes[a][2], icon);
					
					// special locations
					if (String(nodes[a][2]).substr(0, 2) == "wc") {
						nodesWashroom.push(node);
					} else if (String(nodes[a][2]).substr(0, 7) == "parking") {
						nodesParking.push(node);
					} else if (String(nodes[a][2]).substr(0, 6) == "picnic") {
						nodesPicnic.push(node);
					} else if (String(nodes[a][2]).substr(0, 5) == "trash") {
						nodesTrash.push(node);
					}
					
					// intersection exception
					if (String(nodes[a][2]).substr(0, 2) == "i_") { // intersection...
						nodesIntersection.push(node);
					} else { // location...
						nodesLocation.push(node); trace(data);
						if (data != null) {
							//var label:LabelTextField = new LabelTextField(data.name, nodes[a][0], nodes[a][1] + 4)
							//labelsLayer.addChild(label);
							//labelsLayer.graphics.beginFill(CommonStyle.COLOR_TERTIARY);
							//labelsLayer.graphics.drawRect(label.x - 5, label.y, label.textWidth + 10, label.textHeight + 5);
							//labelsLayer.graphics.endFill()							
						}
					}
				} else {
					node = new Node(Graph.getNextIndex(), new Vector3D(nodes[a][0], nodes[a][1]));
				}
				// add node
				node.render();
                graph.addNode(node);
				nodesAll.push(node);
                facilitiesLayer.addChild(node);
            }
			
			//Another loop to create all the edges between nodes
            for(var b:int=0;b<edges.length;b++)
            {
                var from:int = edges[b][0];
                var to:int = edges[b][1];
 
                edge = new Edge(from,to,Vector3D.distance(graph.getNode(from).getPos(),graph.getNode(to).getPos()));
				graph.addEdge(edge);
                //Since the drawEdge method requires the position vectors, we get the nodes
                //from the graph with their index and then get their position with getPos()
                edge.drawEdge(graph.getNode(from).getPos(),graph.getNode(to).getPos());
                facilitiesLayer.addChild(edge);
            }
			
			// add person
			var startingNode:Node = findLocation("parking1");
			personMarker = new PersonMarker(startingNode, uiSpriteSheet.getSprite("marker_person"));
			markersLayer.addChild(personMarker);
			//parkMap.x = startingNode.x;
			//parkMap.y = startingNode.y;
		}
		
		/**
		 * Returns the found location.
		 */
		private function findLocation(id:String):Node 
		{
			var loc:Location;
			for each (loc in nodesLocation) {
				if (loc.id == id) return loc;
			}
			for each (loc in nodesIntersection) {
				if (loc.id == id) return loc;
			}
			return null;
		}
		
	}

}