package ui 
{

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import map.Destination;
	import ui.panel.PanelEvent;
	
	/**
	 * RouteHUD
	 * 
	 * @author Charlie Chao
	 */
	public class RouteHUD extends Sprite 
	{
		/**
		 * Constants
		 */
		public static var EDGE_OFFSET:int = 15;			// offset from edge of screen
		public static var PADDING:int = 10;				// offset from edge of screen
		public static var CONVERSION:Number = 1 / 1.27;	// pixels distance to meters
		public static const ICON_SIZE:int = 48;
		
		/**
		 * Components
		 */
		private var nameText:TextField;
		private var icon:Bitmap;
		//private var distanceText:TextField;
		 
		/**
		 * Attributes
		 */
		private var hudWidth:int;
		private var hudHeight:int;
		
		public function RouteHUD() 
		{
			x = EDGE_OFFSET;
			
			// setup name text
			nameText = new TextField();
			nameText.defaultTextFormat = CommonStyle.ROUTE_HUD_FORMAT_TEXT;
			nameText.autoSize = TextFieldAutoSize.LEFT;
			nameText.selectable = false;
			
			// setup distance text
			//distanceText = new TextField();
			//distanceText.defaultTextFormat = CommonStyle.ROUTE_HUD_FORMAT_TEXT;
			//distanceText.autoSize = TextFieldAutoSize.LEFT;
			//distanceText.selectable = false;
			
			addChild(nameText);
			
			// event listeners
			addEventListener(Event.ADDED_TO_STAGE, added);
			addEventListener(MouseEvent.CLICK, openRoutePlanner);
		}
		
		// added to screen handler
		public function added(e:Event = null):void 
		{
			// reposition
			visible = false;
		}
		
		// opens route planner panel
		public function openRoutePlanner(e:MouseEvent):void 
		{
			dispatchEvent(new PanelEvent(PanelEvent.PANEL_OPEN, { panel: Main.PANEL_ROUTE_PLANNER }));
		}
		
		public function update(dest:Destination = null, iconData:BitmapData = null):void 
		{
			if (dest != null) {
				visible = true;
				
				// setup icon
				if (icon != null && contains(icon)) removeChild(icon);
				icon = new Bitmap(iconData);
				icon.x = (ICON_SIZE - icon.width) / 2 + PADDING;
				icon.y = (ICON_SIZE - icon.height) / 2 + PADDING;
				addChild(icon);
				
				hudHeight = ICON_SIZE + PADDING * 2;
				y = stage.stageHeight - EDGE_OFFSET - hudHeight;
				
				// setup text
				nameText.htmlText = "<b>Current Destination:</b> " + dest.name + "\n" + Math.round(CONVERSION * int(dest.distance)) + "m left";
				nameText.x = icon.x + ICON_SIZE + PADDING;
				nameText.y = (hudHeight - nameText.height) / 2;
				
				// draw background
				graphics.clear();
				graphics.beginFill(CommonStyle.PANEL_COLOR_BG);
				graphics.drawRect(0, 0, nameText.x + nameText.width + PADDING, hudHeight);
				graphics.endFill();
			} else {
				visible = false;
			}
		}
		
	}

}