package ui 
{

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import ui.button.FlatButton;
	import ui.panel.Panel;
	import ui.panel.PanelEvent;
	import com.greensock.TweenLite;
	import com.greensock.easing.Sine;
	
	/**
	 * RouteSlot
	 * 
	 * @author Charlie Chao
	 */
	public class RouteSlot extends Sprite 
	{
		/**
		 * Constants
		 */
		public static const CORNER_SIZE:int = 32;
		public static const HEIGHT:int = 80;
		public static const LINE_HEIGHT:int = 1;
		public static const FADE_DURATION:Number = 0.5;
		
		/**
		 * Statics
		 */
		public static var closeIcon:BitmapData;
		public static var upIcon:BitmapData;
		public static var downIcon:BitmapData;
		
		/**
		 * Components
		 */
		public var indexText:TextField;
		private var label:TextField;
		private var distanceText:TextField;
		private var remove:FlatButton;
		private var moveUp:FlatButton;
		private var moveDown:FlatButton;
		private var icon:Bitmap;
		
		/**
		 * Attributes
		 */
		private var slotWidth:int;
		private var selected:Boolean;
		private var clickBound:Rectangle;
		 
		public function RouteSlot(id:int, labelText:String, distance:Number, width:int, iconData:BitmapData) 
		{
			//x = Panel.PADDING;
			y = (HEIGHT + LINE_HEIGHT) * id + LINE_HEIGHT;
			slotWidth = width;
			selected = (Main.selectedDestination == id);
			clickBound = new Rectangle(CORNER_SIZE, 0, slotWidth - CORNER_SIZE * 2, HEIGHT);
			
			// setup index text
			indexText = new TextField();
			indexText.defaultTextFormat = CommonStyle.INFO_FORMAT_INDEX;
			indexText.autoSize = TextFieldAutoSize.LEFT;
			indexText.selectable = false;
			indexText.text = String(id + 1);
			indexText.x = width - (CORNER_SIZE - indexText.width) / 2 - indexText.width;
			indexText.y = HEIGHT - (CORNER_SIZE - indexText.height) /2 - indexText.height;
			
			// setup label text
			label = new TextField();
			label.defaultTextFormat = CommonStyle.INFO_FORMAT_TEXT_ALT;
			label.autoSize = TextFieldAutoSize.CENTER;
			label.selectable = false;
			label.text = labelText;
			
			// setup distance text
			distanceText = new TextField();
			distanceText.defaultTextFormat = CommonStyle.INFO_FORMAT_TEXT_LARGE;
			distanceText.autoSize = TextFieldAutoSize.CENTER;
			distanceText.selectable = false;
			distanceText.htmlText = String(Math.round(RouteHUD.CONVERSION * distance));
			//distanceText.setTextFormat(CommonStyle.INFO_FORMAT_TEXT_ALT, distanceText.htmlText.length - 2);
			distanceText.defaultTextFormat = CommonStyle.INFO_FORMAT_TEXT_ALT;
			distanceText.htmlText += "m";
			
			// setup icon
			icon = new Bitmap(iconData);
			
			// determine spacing
			var horizontalSpace:int = (width - icon.width) / 2;
			var verticalSpace:int = (HEIGHT - icon.height) / 2;
			
			// re-position elements
			icon.x = CORNER_SIZE + icon.width * 0.1;
			icon.y = verticalSpace;
			if (label != null) {
				label.x = CORNER_SIZE + icon.width * 1.1 + icon.width * 0.3;
				label.y = 16 - 4;
				distanceText.x = label.x;
				distanceText.y = label.y + label.height - 14;
			}
			
			// remove button
			remove = new FlatButton(null, width - CORNER_SIZE, 0, CORNER_SIZE, CORNER_SIZE, closeIcon);
			remove.bgAlpha = 0;
			moveUp = new FlatButton(null, 0, 0, CORNER_SIZE, CORNER_SIZE, upIcon);
			moveUp.bgAlpha = 0;
			moveDown = new FlatButton(null, 0, HEIGHT - CORNER_SIZE, CORNER_SIZE, CORNER_SIZE, downIcon);
			moveDown.bgAlpha = 0;
			remove.bgColor = moveUp.bgColor = moveDown.bgColor = CommonStyle.COLOR_PRIMARY;
			
			remove.listen();
			moveUp.listen();
			moveDown.listen();
			
			// add components
			addChild(indexText);
			addChild(remove);
			addChild(moveUp);
			addChild(moveDown);
			addChild(icon);
			addChild(label);
			addChild(distanceText);
			
			remove.addEventListener(MouseEvent.CLICK, removeSelf);
			moveUp.addEventListener(MouseEvent.CLICK, goUp);
			moveDown.addEventListener(MouseEvent.CLICK, goDown);
			addEventListener(MouseEvent.CLICK, select);
			
			render();
		}
		
		// render
		public function render():void 
		{
			// draw background
			graphics.clear();
			graphics.beginFill((selected)?CommonStyle.COLOR_TERTIARY:CommonStyle.COLOR_PRIMARY);
			graphics.drawRect(0, 0, slotWidth, HEIGHT);
			graphics.endFill();
			graphics.beginFill(CommonStyle.COLOR_SECONDARY);
			graphics.drawRect(0, HEIGHT, slotWidth, LINE_HEIGHT);
			graphics.endFill();
			graphics.beginFill(CommonStyle.COLOR_SECONDARY);
			graphics.drawRect(0, -LINE_HEIGHT, slotWidth, LINE_HEIGHT);
			graphics.endFill();
			
			// draw bottm-right corner
			graphics.beginFill(CommonStyle.COLOR_SECONDARY);
			graphics.lineStyle();
			graphics.drawRect(slotWidth - CORNER_SIZE, HEIGHT - CORNER_SIZE, CORNER_SIZE, CORNER_SIZE);
			graphics.endFill();
		}
		
		// get id
		public function get id():int
		{
			return int(indexText.text) - 1;
		}
		
		// go up handler
		public function goUp(e:Event = null):void 
		{
			dispatchEvent(new PanelEvent(PanelEvent.PANEL_CHANGE, "goUp"));
		}
		
		// go down handler
		public function goDown(e:Event = null):void 
		{
			dispatchEvent(new PanelEvent(PanelEvent.PANEL_CHANGE, "goDown"));
		}
		
		// remove handler
		public function removeSelf(e:Event = null):void 
		{
			dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Destination Removed", "'" + label.text + "' has been removed from route list."));
			TweenLite.to(this, FADE_DURATION, { alpha: 0, x: width, ease: Sine.easeOut, onComplete: dispose } );
		}
		
		// toggle selection
		public function select(e:MouseEvent):void 
		{
			if (clickBound.containsPoint(new Point(mouseX, mouseY))) {
				dispatchEvent(new PanelEvent(PanelEvent.PANEL_CHANGE, (selected)?"deselect":"select"));
				render();
			}
		}
		
		// dispose
		public function dispose():void 
		{
			remove.unlisten();
			moveUp.unlisten();
			moveDown.unlisten();
			moveUp.removeEventListener(MouseEvent.CLICK, goUp);
			moveDown.removeEventListener(MouseEvent.CLICK, goDown);
			removeEventListener(MouseEvent.CLICK, select);
			dispatchEvent(new PanelEvent(PanelEvent.PANEL_CHANGE, null));
		}
		
	}

}