package ui 
{
	import flash.display.DisplayObject;
	import flash.display.Shader;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.events.TransformGestureEvent;
	import flash.geom.Point;
	import com.greensock.TweenLite;
	import com.greensock.easing.Sine;
	import ui.button.InfoButton;
	import ui.panel.Panel;

	/**
	 * ScrollContainer
	 * 
	 * @author Charlie Chao
	 */
	public class ScrollContainer extends Sprite implements IListenable
	{
		// constants
		public static const SCROLL_SPEED:int = 40;			// scroll speed in pixels
		public static const DURATION_SCROLL:Number = 0.2;	// smooth scroll animation duration
		public static const DURATION_FADE:Number = 0.1;		// fade animation duration	
		
		// attributes
		public var maxHeight:int;			// maximum height the container can scroll to
		public var maskContainer:Shape;		// mask layer
		public var container:Sprite;		// content container
		
		public function ScrollContainer() 
		{
			super();
			
			// setup mask
			maskContainer = new Shape();
			
			// setup container
			container = new Sprite();
			container.mask = maskContainer;
			
			// add components
			addChild(maskContainer);
			addChild(container);
		}
		
		// attaches scroll listener
		public function listen():void 
		{
			// add linked panel button listeners
			for (var i:int = 0; i < container.numChildren; i++) {
				var child:DisplayObject = container.getChildAt(i); 
				if (child is IListenable) IListenable(child).listen();
			}
			parent.addEventListener(MouseEvent.MOUSE_WHEEL, scrollMouse);
			parent.addEventListener(TransformGestureEvent.GESTURE_SWIPE, scrollTouch);
		}
		
		// detaches scroll listener
		public function unlisten():void 
		{
			// add linked panel button listeners
			for (var i:int = 0; i < container.numChildren; i++) {
				var child:DisplayObject = container.getChildAt(i);
				if (child is IListenable) IListenable(child).unlisten();
			}
			container.y = 0;		// resets container position
			parent.removeEventListener(MouseEvent.MOUSE_WHEEL, scrollMouse);
			parent.addEventListener(TransformGestureEvent.GESTURE_SWIPE, scrollTouch);
		}
		
		// scroll handler
		public function scrollMouse(e:MouseEvent):void 
		{
			scroll(e.delta);
		}
		
		// scroll swipe handler
		public function scrollTouch(e:TransformGestureEvent):void 
		{
			scroll(e.offsetY);
		}
		
		// scroll
		public function scroll(delta:int):void 
		{
			TweenLite.killTweensOf(container);					// kill previous tweens
			var dist:int = container.y + delta * SCROLL_SPEED;	// projected distance to scroll
			maxHeight = Math.max(0, container.height - maskContainer.height + Panel.PADDING * 4);
			if (dist > 0) { // container is too high...
				TweenLite.to(container, DURATION_SCROLL, { y: dist, onComplete: elasticTop } );
			} else if (dist < -maxHeight) { // container is too low...
				TweenLite.to(container, DURATION_SCROLL, { y: dist, onComplete: elasticBottom } );
			} else {
				// smooth scroll
				TweenLite.to(container, DURATION_SCROLL, { y: dist} );
			}
		}
		
		// top elastic effect
		public function elasticTop():void {
			TweenLite.to(container, DURATION_SCROLL, { y: 0, ease: Sine.easeOut } );
		}
		
		// bottom elastic effect
		public function elasticBottom():void {
			TweenLite.to(container, DURATION_SCROLL, { y: -maxHeight, ease: Sine.easeOut } );
		}
		
		// adds object to inner container
		public function add(child:DisplayObject):void { container.addChild(child); }
		
		// removes object from inner container
		public function remove(child:DisplayObject):void { container.removeChild(child); }
		
		// clear all objects from inner container
		public function clear():void {
			container.removeChildren();
			container.y = 0;
		}
		
		// refresh container with new contents
		public function refresh(list:Array, animate:Boolean = true):void 
		{
			unlisten();
			if (animate) TweenLite.to(container, DURATION_FADE, { alpha: 0, onComplete: removeAndAdd, onCompleteParams: [list, true] } );
			else removeAndAdd(list);
		}
		
		// transition effect for refresh()
		private function removeAndAdd(list:Array = null, animate:Boolean = true):void {
			clear();
			for each (var obj:DisplayObject in list[0]) {
				container.addChild(obj);
			}
			listen();
			TweenLite.to(container, DURATION_FADE, { alpha: 1 } );
		}
		
		// sets mask size
		public function set bound(size:Point):void 
		{
			maskContainer.graphics.clear();
			maskContainer.graphics.beginFill(0x000000);
			maskContainer.graphics.drawRect(0, 0, size.x, size.y);
			maskContainer.graphics.endFill();
		}
		
	}

}