package ui
{
	import flash.display.BitmapData;
	import flash.display.Bitmap;
	import flash.display.Loader;
	import flash.display.LoaderInfo;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IOErrorEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	/**
	 * Spritesheet
	 * @author Charlie Chao
	 */
	public class Spritesheet extends EventDispatcher
	{
		/**
		 * Constants
		 */
		public static const PLACEHOLDER_SIZE:int = 48;
		
		// attributes
		private var jsonLoader:URLLoader;
		private var jsonData:Object;
		private var imageLoader:Loader;
		private var source:String;
		private var sprites:Vector.<SpriteData>;
		private var placeholder:BitmapData;
		
		// constructor
		public function Spritesheet(source:String):void
		{
			this.source = source;
			
			// create placeholder
			placeholder = new BitmapData(PLACEHOLDER_SIZE, PLACEHOLDER_SIZE, false, CommonStyle.PLACEHOLDER_COLOR);
			
			// load spritesheet
			jsonLoader = new URLLoader(); 
			jsonLoader.addEventListener(Event.COMPLETE, jsonLoaded);
			jsonLoader.load(new URLRequest(source));
		}
		
		// loads json data
		private function jsonLoaded(e:Event):void 
		{
			jsonLoader.removeEventListener(Event.COMPLETE, jsonLoaded);
			// get json data
			var data:Object = JSON.parse(String(URLLoader(jsonLoader).data));
			jsonData = data.frames;			
			
			// close json loader
			jsonLoader.close();
			jsonLoader = null;
			
			// start image loader
			imageLoader = new Loader();
			imageLoader.contentLoaderInfo.addEventListener(Event.COMPLETE, processImage);
			imageLoader.load(new URLRequest(source.replace(".json", ".png")));
		}
		
		// loads image data and fill in 
		private function processImage(e:Event):void 
		{
			imageLoader.contentLoaderInfo.removeEventListener(Event.COMPLETE, processImage);
			// get image data
			var data:BitmapData = Bitmap(LoaderInfo(e.target).content).bitmapData;
			
			// extract sprites
			sprites = new Vector.<SpriteData>;
			var spriteName:String;
			var bitmap:BitmapData;
			for each (var o:Object in jsonData) {
				spriteName = String(o.filename).replace("0000", "");
				bitmap = new BitmapData(o.frame.w, o.frame.h);
				bitmap.copyPixels(data, new Rectangle(o.frame.x, o.frame.y, o.frame.w, o.frame.h), new Point(0, 0));
				sprites.push(new SpriteData(spriteName, bitmap));
			}
			
			// close image loader
			jsonData = null;
			imageLoader = null;
			
			// alert app that the spritesheet is ready
			dispatchEvent(new StateEvent(StateEvent.READY));
		}
		
		// gets sprite bitmap by name
		public function getSprite(spriteName:String):BitmapData
		{
			if (spriteName != null && spriteName != "") {
				for (var i:int = 0; i < sprites.length; i++) {
					if (sprites[i].name == spriteName) return sprites[i].data;
				}
			}
			// return placeholder
			return placeholder;
		}
		
	}

}