package ui 
{

	import com.greensock.core.TweenCore;
	import com.greensock.TimelineLite;
	import com.greensock.TweenLite;
	import com.greensock.easing.Sine;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	
	/**
	 * Toast
	 * Toast notification for quick info tips.
	 * @author Charlie Chao
	 */
	public class Toast extends Sprite 
	{
		/**
		 * Constants
		 */
		public static const WIDTH_PERCENTAGE:Number = 0.75;		// toast width based on % of screen width
		public static const DURATION:Number = 3;				// duration on-screen
		public static const PADDING:int = 10;				// duration on-screen
		public static const BAR_HEIGHT:int = 4;				// duration on-screen
		
		/**
		 * Components
		 */
		public var titleText:TextField;
		public var messageText:TextField;
		
		/**
		 * Attributes
		 */
		private var toastWidth:int;
		private var toastHeight:int;
		private var duration:int;
		private var tween:TimelineLite;
		
		public function Toast() 
		{
			super();
			
			// setup title
			titleText = new TextField();
			titleText.x = PADDING;
			titleText.y = PADDING;
			titleText.defaultTextFormat = CommonStyle.TOAST_FORMAT_TITLE;
			titleText.autoSize = TextFieldAutoSize.LEFT;
			titleText.multiline = true;
			titleText.wordWrap = true;
			titleText.selectable = false;
			
			// setup message
			messageText = new TextField();
			messageText.x = PADDING;
			messageText.defaultTextFormat = CommonStyle.TOAST_FORMAT_MSG;
			messageText.autoSize = TextFieldAutoSize.LEFT;
			messageText.multiline = true;
			messageText.wordWrap = true;
			messageText.selectable = false;
			
			// setup animation tween
			tween = new TimelineLite({ easing: Sine.easeIn, onUpdate: drawBar, onComplete: fadeOut });
			tween.append(new TweenLite(this, DURATION / 6, { y: 0 }));
			tween.append(new TweenLite(this, DURATION, { }));
			tween.stop();
			
			// add component
			addChild(titleText);
			addChild(messageText);
		}
		
		// draws duration bar
		public function drawBar():void 
		{
			// draw background
			graphics.clear();
			graphics.beginFill(CommonStyle.TOAST_COLOR_BG);
			graphics.drawRect(0, 0, toastWidth, toastHeight);
			graphics.endFill();
			
			// draw bar
			graphics.beginFill(CommonStyle.TOAST_COLOR_BAR);
			graphics.drawRect(0, 0, toastWidth * tween.currentProgress, BAR_HEIGHT);
			graphics.endFill();
		}
		
		// updates metrics
		public function update():void 
		{
			toastWidth = stage.stageWidth * WIDTH_PERCENTAGE;
			titleText.width = messageText.width = toastWidth - PADDING * 2;
			x = (stage.stageWidth - toastWidth) / 2;
		}
		
		// shows toast
		public function showToast(title:String, msg:String):void 
		{
			update();
			titleText.text = title;
			messageText.text = msg;
			messageText.y = titleText.y + titleText.textHeight + PADDING;
			toastHeight = messageText.y + messageText.height + PADDING;
			visible = true;
			alpha = 1;
			y = -height;
			parent.setChildIndex(this, parent.numChildren - 1);
			tween.restart();
			addEventListener(MouseEvent.CLICK, fadeOut);
			addEventListener(MouseEvent.ROLL_OVER, pause);
			addEventListener(MouseEvent.ROLL_OUT, resume);
		}
		
		// pause
		public function pause(e:MouseEvent = null):void
		{
			tween.pause();
		}
		
		// resume
		public function resume(e:MouseEvent = null):void
		{
			tween.resume();
		}
		
		// fade out
		public function fadeOut(e:MouseEvent = null):void
		{
			tween.stop();
			TweenLite.to(this, DURATION / 6, { alpha: 0, onComplete: hide } );
		}
		
		// hide
		public function hide(e:MouseEvent = null):void 
		{
			visible = false;
			removeEventListener(MouseEvent.CLICK, hide);
			removeEventListener(MouseEvent.ROLL_OVER, pause);
			removeEventListener(MouseEvent.ROLL_OUT, resume);
		}
	}

}