package ui.button
{

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import ui.IListenable;
	
	/**
	 * FlatButton
	 * 
	 * @author Charlie Chao
	 */
	public class FlatButton extends Sprite implements IListenable
	{
		// attributes
		public var label:TextField;
		public var icon:Bitmap;
		public var buttonWidth:int;
		public var buttonHeight:int;
		public var bgColor:uint;
		public var bgColorDown:uint;
		public var bgAlpha:Number;
		public var bgAlphaDown:Number;
		
		public function FlatButton(labelText:String, x:int, y:int, width:int, height:int, iconData:BitmapData): void
		{
			this.x = x;
			this.y = y;
			buttonWidth = width;
			buttonHeight = height;
			bgColor = CommonStyle.BUTTON_COLOR_BG;
			bgColorDown = CommonStyle.BUTTON_COLOR_BG_DOWN;
			bgAlpha = bgAlphaDown = 1;
				
			// setup icon
			icon = new Bitmap(iconData);
			addChild(icon);
			
			// setup label if exists
			if (labelText != null && labelText != "") {
				label = new TextField();
				label.defaultTextFormat = CommonStyle.BUTTON_FORMAT_LABEL;
				label.autoSize = TextFieldAutoSize.LEFT;
				label.width = buttonWidth;
				label.multiline = true;
				label.wordWrap = true;
				label.selectable = false;
				//label.embedFonts = true;
				label.text = labelText;
				addChild(label);
			}
			
			// add initial render event
			addEventListener(Event.ADDED_TO_STAGE, render);
		}
		
		// added to screen handler
		protected function added(e:Event = null):void 
		{
			//listen();
			render();
		}
		
		// draws button
		public function render(e:Event = null):void
		{
			// remove initial render event
			if (hasEventListener(Event.ADDED)) removeEventListener(Event.ADDED, render);
			
			// draw base
			graphics.clear();
			drawBackground();
			
			// determine spacing
			var horizontalSpace:int = (buttonWidth - icon.width) / 2;
			var verticalSpace:int = (buttonHeight - icon.height - ((label != null)?label.height:0)) / ((label != null)?3:2);
			
			// re-position elements
			icon.x = (horizontalSpace < 0)?0:horizontalSpace;
			icon.y = (verticalSpace < 0)?0:verticalSpace;
			if (label != null) {
				label.x = buttonWidth / 2 - label.width / 2;
				label.y = icon.y + icon.height + verticalSpace;
				label.width = buttonWidth;// - horizontalSpace * 2;
			}
		}
		
		// draw button background
		protected function drawBackground(down:Boolean = false):void 
		{
			//graphics.clear();
			//graphics.beginFill((down)?bgColorDown:bgColor, (down)?bgAlphaDown:bgAlpha);
			//graphics.drawRect(0, 0, buttonWidth, buttonHeight);
			//graphics.endFill();
			
			graphics.clear();
			graphics.beginFill((down)?bgColorDown:bgColor, (down)?bgAlphaDown:bgAlpha);
			//graphics.drawRect(0, 0, buttonWidth, buttonHeight - 16);
			//graphics.drawRect(16, buttonHeight-16, buttonWidth-16, 16);
			graphics.drawRoundRectComplex(0, 0, buttonWidth, buttonHeight, 0, 0, 0, 8);
			graphics.endFill();
		}
		
		// button state handler
		protected function stateDown(e:MouseEvent = null):void { drawBackground(true); }
		protected function stateUp(e:MouseEvent = null):void { drawBackground(); }
		
		// attachs button state listener
		public function listen():void {
			addEventListener(MouseEvent.MOUSE_DOWN, stateDown);
			addEventListener(MouseEvent.MOUSE_UP, stateUp);
			addEventListener(MouseEvent.MOUSE_OUT, stateUp);
		}
		
		// detaches button state listener
		public function unlisten():void {
			removeEventListener(MouseEvent.MOUSE_DOWN, stateDown);
			removeEventListener(MouseEvent.MOUSE_UP, stateUp);
			removeEventListener(MouseEvent.MOUSE_OUT, stateUp);
		}
		
		// returns label text as name
		override public function get name():String 
		{
			return label.text;
		}
	}

}