package ui.button 
{

	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import map.MapEvent;
	import ui.panel.Panel;
	
	/**
	 * LayersButton
	 * 
	 * @author Charlie Chao
	 */
	public class LayersButton extends FlatButton 
	{
		/**
		 * Constants
		 */
		public static const BUTTON_HEIGHT:int = 48;
		
		/**
		 * Components
		 */
		public var iconActive:Bitmap;
		
		public function LayersButton(labelText:String, x:int, y:int, width:int, iconDataInactive:BitmapData, iconDataActive:BitmapData) 
		{
			super(labelText, x, y, width, BUTTON_HEIGHT, iconDataInactive);
			bgColor = CommonStyle.SIDE_BUTTON_COLOR_BG;
			bgColorDown = CommonStyle.SIDE_BUTTON_COLOR_BG_DOWN;
			label.defaultTextFormat = CommonStyle.LAYERS_BUTTON_FORMAT_LABEL;
			label.text = labelText;
			
			// setup active icon
			iconActive = new Bitmap(iconDataActive);
			iconActive.visible = false;
			addChild(iconActive);
			
			// activate layer by default
			active = true;
		}
		
		// renders the button
		override public function render(e:Event = null):void 
		{
			super.render(e);
			
			// determine spacing
			var space:int = (buttonHeight - icon.height) / 2;
			
			// re-position elements
			icon.x = iconActive.x = (space < 0)?0:space;
			icon.y = iconActive.y = (space < 0)?0:space;
			if (label != null) {
				label.x = Panel.PADDING + icon.width;
				label.y = buttonHeight / 2 - label.height / 2;
			}
		}
			
		/**
		 * Attachs button state listener
		 */
		override public function listen():void 
		{
			super.listen();
			addEventListener(MouseEvent.CLICK, toggle);
		}
		
		/**
		 * Toggles layer state.
		 */
		private function toggle(e:MouseEvent):void 
		{
			active = !active;
		}
		
		/**
		 * Sets layer state.
		 */
		public function set active(value:Boolean):void
		{
			iconActive.visible = value; // shows active icon
			dispatchEvent(new MapEvent(MapEvent.LAYER_TOGGLE, { name: name, state: value }));
		}
		
		/**
		 * Returns layer state.
		 */
		public function get active():Boolean
		{
			return iconActive.visible;
		}
		
	}

}