package ui.button
{
	
	import com.greensock.TweenLite;
	import flash.display.BitmapData;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import ui.panel.Panel;
	import ui.panel.PanelEvent;

	/**
	 * SideButton
	 * A side button for quick access.
	 * @author Charlie Chao
	 */
	public class SideButton extends LinkedPanelButton 
	{
		
		// constants
		public static const BAR_WIDTH:int = 6;
		public static const BUTTON_SIZE:int = 50;
		public static const BUTTON_SPACING:int = 10;
		
		// attributes
		private var active:Boolean;
		
		// animation
		private var tween:TweenLite;
		public var barX:int = 0;
		
		public function SideButton(x:int, y:int, width:int, height:int, iconData:BitmapData, linkedPanel:int) 
		{
			super(null, x, y, width, height, iconData, linkedPanel);
			bgColor = CommonStyle.SIDE_BUTTON_COLOR_BG;
			bgColorDown = CommonStyle.SIDE_BUTTON_COLOR_BG_DOWN;
			
			// setup animation
			tween = new TweenLite(this, 0.2, {barX: -BAR_WIDTH, onUpdate: updateBar});//hexColors:{bgColor: bgColorDown}, 
			tween.paused = true;
			
			// add event listeners
			listen();
		}
		
		// opens panel
		override protected function openPanel(e:MouseEvent):void 
		{
			if (!active) setState(true);
			dispatchEvent(new PanelEvent(PanelEvent.PANEL_OPEN, { panel: linkedPanel }));
		}
		
		// updates state bar
		private function updateBar():void 
		{
			graphics.clear(); 
			graphics.beginFill(CommonStyle.SIDE_BUTTON_COLOR_BAR);
			graphics.drawRoundRectComplex(barX, 0, BAR_WIDTH + 10, buttonHeight, 0, 0, 8, 0);
			graphics.endFill();
			drawBackground();
		}
		
		override protected function drawBackground(down:Boolean = false):void 
		{
			graphics.beginFill((down)?bgColorDown:bgColor, (down)?bgAlphaDown:bgAlpha);
			//graphics.drawRect(0, 0, buttonWidth, buttonHeight - 16);
			//graphics.drawRect(16, buttonHeight-16, buttonWidth-16, 16);
			graphics.drawRoundRectComplex(0, 0, buttonWidth, buttonHeight, 0, 0, 8, 0);
			graphics.endFill();
		}
		
		// sets button state
		public function setState(active:Boolean):void 
		{
			this.active = active;
			if (this.active) tween.restart();
			else tween.reverse();
		}
		
	}

}