package ui.panel
{
	import flash.events.Event;
	import map.MapData;
	import ui.button.InfoButton;
	import ui.button.LinkedPanelButton;
	import ui.Spritesheet;

	/**
	 * ActivitiesPanel
	 * 
	 * @author Charlie Chao
	 */
	public class ActivitiesPanel extends Panel 
	{
		/**
		 * Constants
		 */
		public static const WIDTH:int = 300;
		public static const BUTTONS_HEIGHT:int = 160;
		
		/**
		 * Components
		 */
		public var activitiesButton:Vector.<InfoButton>;
		
		/**
		 * Models
		 */
		private var activities:Object;				// activities reference
		
		public function ActivitiesPanel(mapData:MapData) 
		{
			super(WIDTH, Main.CATEGORY_MAIN_MENU, "main_menu_activities", "activities");
			title.text = "Activities";
			x = 0; // makes the panel fades in only
			activities = mapData.activities;
			
			// add initial render event
			addEventListener(Event.ADDED_TO_STAGE, added);
		}
		
		// initializes the buttons
		override public function constructButtons(panels:Vector.<Panel>):void
		{
			// add buttons
			activitiesButton = new Vector.<InfoButton>;
			populateButtons(activitiesButton);
			activities = null; // removes reference
			
			// add to screen
			for each (var a:InfoButton in activitiesButton) {
				content.add(a);
			}
			
			// add scrollable content container
			addChild(content);
		}
		
		// populate buttons in two columns
		public function populateButtons(buttonsList:Vector.<InfoButton>):void 
		{
			var buttonWidth:int = (WIDTH - PADDING * 3) / 2;
			var buttonX:int, buttonY:int;
			var left:Boolean = true;
			for (var i:int = 0; i < activities.length; i++) {
				// fill buttons in two columns
				buttonX = (left)?(PADDING):(PADDING * 2 + buttonWidth);
				buttonY = PADDING + int(i/2) * (BUTTONS_HEIGHT + PADDING);
				buttonsList.push(new InfoButton(activities[i].name, buttonX, buttonY, buttonWidth, BUTTONS_HEIGHT, uiSpriteSheet.getSprite(activities[i].icon), activities[i].id));
				left = !left; // column swap
			}
		}
		
	}

}