package ui.panel
{
	import flash.events.Event;
	import flash.events.MouseEvent;
	import map.MapData;
	import ui.button.FlatButton;
	import ui.button.InfoButton;
	import ui.Spritesheet;

	/**
	 * FacilitiesPanel
	 * 
	 * @author Charlie Chao
	 */
	public class FacilitiesPanel extends Panel 
	{
		/**
		 * Constants
		 */
		public static const WIDTH:int = 300;
		public static const BUTTONS_WIDTH:int = (WIDTH - PADDING * 3) / 2;
		public static const BUTTONS_HEIGHT:int = 140;
		
		/**
		 * Components
		 */
		public var categoriesButtons:Vector.<FlatButton>;
		public var generalButtons:Vector.<InfoButton>;
		public var recButtons:Vector.<InfoButton>;
		public var sightseeingButtons:Vector.<InfoButton>;
		public var transportationButtons:Vector.<InfoButton>;
		
		/**
		 * Models
		 */
		private var locations:Object;				// location data reference
		public var general:Vector.<Object>;
		public var rec:Vector.<Object>;
		public var sightseeing:Vector.<Object>;
		public var transportation:Vector.<Object>;
		
		public function FacilitiesPanel(mapData:MapData) 
		{
			super(WIDTH, Main.CATEGORY_MAIN_MENU, "main_menu_facilities", "poi"); // TODO: image
			title.text = "Points of Interest";
			x = 0; // makes the panel fades in only
			locations = mapData.locations;
			
			// add initial render event
			addEventListener(Event.ADDED_TO_STAGE, added);
		}
		
		// initializes the buttons
		override public function constructButtons(panels:Vector.<Panel>):void
		{
			// setup models
			general = new Vector.<Object>;
			rec = new Vector.<Object>;
			sightseeing = new Vector.<Object>;
			transportation = new Vector.<Object>;
			for (var i:int = 0; i < locations.length; i++) {
				var ref:Vector.<Object> = this[locations[i].category] as Vector.<Object>;
				ref.push( { name: locations[i].name, icon: locations[i].icon, id:locations[i].id } );
			}
			locations = null;
			
			// add buttons
			categoriesButtons = new Vector.<FlatButton>;
			categoriesButtons.push(new FlatButton("General", 0, 0, BUTTONS_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite("poi_cat_general")));
			categoriesButtons.push(new FlatButton("Recreation", 0, 0, BUTTONS_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite("poi_cat_recreation")));
			categoriesButtons.push(new FlatButton("Sightseeing", 0, 0, BUTTONS_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite("poi_cat_sightseeing")));
			categoriesButtons.push(new FlatButton("Transportation", 0, 0, BUTTONS_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite("poi_cat_transportation")));
			positionButtons([categoriesButtons]);
			
			generalButtons = new Vector.<InfoButton>;
			populateButtons(generalButtons, general);
			positionButtons([generalButtons]);
			
			recButtons = new Vector.<InfoButton>;
			populateButtons(recButtons, rec);
			positionButtons([recButtons]);
			
			sightseeingButtons = new Vector.<InfoButton>;
			populateButtons(sightseeingButtons, sightseeing);
			positionButtons([sightseeingButtons]);
			
			transportationButtons = new Vector.<InfoButton>;
			populateButtons(transportationButtons, transportation);
			positionButtons([transportationButtons]);
			
			// delete models
			general.length = rec.length = sightseeing.length = transportation.length = 0;
			
			// add scrollable content container
			addChild(content);
		}
		
		// added to screen handler
		override public function added(e:Event = null):void 
		{
			super.added(e);
			categoriesButtons[0].addEventListener(MouseEvent.CLICK, showGeneral);
			categoriesButtons[1].addEventListener(MouseEvent.CLICK, showRec);
			categoriesButtons[2].addEventListener(MouseEvent.CLICK, showSightseeing);
			categoriesButtons[3].addEventListener(MouseEvent.CLICK, showTransportation);
			backButton.removeEventListener(MouseEvent.CLICK, back);
			content.refresh([categoriesButtons], false); // resets buttons display
		}
		
		// removes the panel from screen and removes all the listeners
		override protected function dispose():void 
		{
			super.dispose();
			categoriesButtons[0].removeEventListener(MouseEvent.CLICK, showGeneral);
			categoriesButtons[1].removeEventListener(MouseEvent.CLICK, showRec);
			categoriesButtons[2].removeEventListener(MouseEvent.CLICK, showSightseeing);
			categoriesButtons[3].removeEventListener(MouseEvent.CLICK, showTransportation);
		}
		
		// populates buttons list
		private function populateButtons(buttonsList:Vector.<InfoButton>, buttonsModel:Vector.<Object>):void 
		{
			for (var i:int = 0; i < buttonsModel.length; i++) {
				buttonsList.push(new InfoButton(buttonsModel[i].name, 0, 0, BUTTONS_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite(buttonsModel[i].icon), buttonsModel[i].id));
			}
		}
		
		// positions buttons in two columns
		private function positionButtons(buttonsList:Array):void 
		{
			var left:Boolean = true;
			for (var i:int = 0; i < buttonsList[0].length; i++) {
				// fill buttons in two columns
				buttonsList[0][i].x = (left)?(PADDING):(PADDING * 2 + BUTTONS_WIDTH);
				buttonsList[0][i].y = PADDING + int(i / 2) * (BUTTONS_HEIGHT + PADDING);
				left = !left; // column swap
			}
		}
		
		// shows categories
		private function showGeneral(e:MouseEvent = null):void { content.refresh([generalButtons]); backButton.addEventListener(MouseEvent.CLICK, back, false, 1); }
		private function showRec(e:MouseEvent = null):void { content.refresh([recButtons]); backButton.addEventListener(MouseEvent.CLICK, back, false, 1); }
		private function showSightseeing(e:MouseEvent = null):void { content.refresh([sightseeingButtons]); backButton.addEventListener(MouseEvent.CLICK, back, false, 1); }
		private function showTransportation(e:MouseEvent = null):void { content.refresh([transportationButtons]); backButton.addEventListener(MouseEvent.CLICK, back, false, 1); }
		
		// changes back button to return to show categories
		private function back(e:MouseEvent = null):void {
			e.stopImmediatePropagation();
			backButton.removeEventListener(MouseEvent.CLICK, back);
			content.refresh([categoriesButtons]);
		}
		
	}

}