package ui.panel 
{
	import flash.display.Bitmap;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import map.MapData;
	import map.MapEvent;
	import ui.button.LongButton;
	import ui.RouteEvent;
	import ui.Spritesheet;
	import com.greensock.TweenLite;
	import ui.ToastEvent;

	/**
	 * InfoPanel
	 * 
	 * @author Charlie Chao
	 */
	public class InfoPanel extends Panel 
	{
		/**
		 * Constants
		 */
		public static const WIDTH:int = 300;
		public static const CONTENT_WIDTH:int = WIDTH - PADDING * 2;
		public static const BUTTONS_HEIGHT:int = 48;
		public static const CONTENT_OFFSET:int = PADDING * 3 + BUTTONS_HEIGHT * 2;	// for locations item
		public static const TOAST_INSERT_BEFORE:String = " is now set as your current destination."
		public static const TOAST_INSERT_AFTER:String = " has been added to the end of your route list."
		
		/**
		 * Attributes
		 */
		private var currentData:Object;
		private var currentID:String;
		private var previousIDs:Vector.<String>;	// for back button
		private var panelData:Array;				// map data reference
		
		/**
		 * Components
		 */
		 private var contentText:TextField;
		 private var btnGoTo:LongButton;
		 private var btnAddQueue:LongButton;
		 private var btnLocations:Vector.<LongButton>;		// relevant locations (for activities)
		
		public function InfoPanel(mapData:MapData) 
		{
			super(WIDTH, Main.CATEGORY_MAIN_MENU, null, "placeholder"); // TODO: icon
			title.text = "Information";
			x = 0; // makes the panel fades in only
			previousIDs = new Vector.<String>;
			
			// get panel data and combine them
			panelData = mapData.activities as Array;
			panelData = panelData.concat(mapData.locations as Array);
			
			// setup textfield
			contentText = new TextField();
			contentText.x = PADDING;
			contentText.defaultTextFormat = CommonStyle.INFO_FORMAT_TEXT;
			contentText.autoSize = TextFieldAutoSize.LEFT;
			contentText.width = CONTENT_WIDTH;
			contentText.multiline = true;
			contentText.wordWrap = true;
			
			// setup buttons
			btnGoTo = new LongButton("Set Desination", PADDING, PADDING, CONTENT_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite("panel_atd"));
			btnGoTo.visible = false;
			btnAddQueue = new LongButton("Add to Route List", PADDING, PADDING * 2 + BUTTONS_HEIGHT, CONTENT_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite("panel_atq"));
			btnAddQueue.visible = false;
			btnLocations = new Vector.<LongButton>;
			
			// add to screen
			content.add(btnGoTo);
			content.add(btnAddQueue);
			content.add(contentText);
			
			addChild(content);
		}
		
		// added to screen handler
		override public function added(e:Event = null):void 
		{
			super.added(e);
			currentID = null;
			previousIDs.length = 0;
		}
		
		// returns an item from panel data
		public function getItem(id:String):Object
		{
			for (var i:int = 0; i < panelData.length; i++) {
				if (panelData[i].id == id) {
					return panelData[i];
				}
			}
			return null;
		}
		
		// loads info and updates panel (button event)
		private function loadInfoMouse(e:MouseEvent = null):void
		{
			var target:LongButton = LongButton(e.currentTarget);
			loadInfo(target.infoID);
			target.removeEventListener(MouseEvent.CLICK, loadInfoMouse);
		}
		
		// resets UI contents
		private function reset():void 
		{
			// reset panel
			contentText.y = PADDING;
			btnGoTo.visible = btnAddQueue.visible = false;
			contentText.htmlText = "";
			
			// remove button events
			btnGoTo.addEventListener(MouseEvent.CLICK, insertBefore);
			btnAddQueue.addEventListener(MouseEvent.CLICK, insertAfter);
			
			// remove relevant locations buttons
			if (btnLocations.length > 0) {
				for each (var b:LongButton in btnLocations) {
					b.unlisten();
					b.addEventListener(MouseEvent.CLICK, loadInfoMouse);
					content.remove(b);
					btnLocations.splice(btnLocations.indexOf(b), 1);
				}
			}
		}
		
		// loads info and updates panel
		public function loadInfo(id:String, goingBack:Boolean = false):void
		{
			// check existence of panel data
			if (getItem(id) != null) { // data found...
				// refresh
				if (currentID != null) { 
					// set previous info panel item
					if (!goingBack) previousIDs.push(currentID);
					if (previousIDs.length == 1) backButton.addEventListener(MouseEvent.CLICK, back, false, 1);
					currentID = id;
					
					// fade out
					TweenLite.killTweensOf(content);
					TweenLite.to(content, 0.2, { alpha: 0, onComplete: refresh });
				} else {
					currentID = id;
					refresh(false);
				}
			} else { // data not found...
				reset();
				title.text = "Error! OMGASH!";
				contentText.htmlText = "404";
			}
		}
		
		// refresh UI
		private function refresh(animate:Boolean = true):void 
		{
			// reset panel
			reset();

			// get panel data
			currentData = getItem(currentID);
			// update panel content
			title.text = currentData.name;
			contentText.htmlText = currentData.desc;
			removeChild(icon); 
			icon = new Bitmap(uiSpriteSheet.getSprite(String(currentData.icon) + "_small")); addChild(icon);
			icon.x = Panel.PADDING;
			icon.y = Panel.TITLE_OFFSET + (Panel.TITLE_HEIGHT - icon.height) / 2;
			if (currentData.img != undefined) {
				if (contains(image)) removeChild(image);
				loadImage(currentData.img);
				setChildIndex(image, 0);
			}
			
			if (currentData.locations != undefined) { // is an activity item...
				// generate relevant locations buttons
				for (var i:int = 0; i < currentData.locations.length; i++) {
					var location:Object = getItem(currentData.locations[i]);
					if (location != null) {
						var tempButton:LongButton = new LongButton(location.name, PADDING, PADDING + i * (PADDING + BUTTONS_HEIGHT), CONTENT_WIDTH, BUTTONS_HEIGHT, uiSpriteSheet.getSprite(location.icon + "_small_alt"), location.id);
						btnLocations.push(tempButton);
						tempButton.addEventListener(MouseEvent.CLICK, loadInfoMouse);
						content.add(tempButton);
					}
				}
				if (btnLocations.length > 0) contentText.y = btnLocations[btnLocations.length - 1].y + BUTTONS_HEIGHT + PADDING;
			} else { // is an location item...
				btnGoTo.visible = btnAddQueue.visible = true;
				contentText.y = CONTENT_OFFSET;
				btnGoTo.addEventListener(MouseEvent.CLICK, insertBefore);
				btnAddQueue.addEventListener(MouseEvent.CLICK, insertAfter);
			}
			
			// dispatch panel change event
			//trace(currentData.id);
			dispatchEvent(new MapEvent(MapEvent.CHANGE, currentData.id));
			
			// fade in
			if (animate) {
				TweenLite.killTweensOf(content);
				TweenLite.to(content, 0.2, { alpha: 1 } );
			}
		}
		
		// changes back button to return to show categories
		private function back(e:MouseEvent = null):void {
			e.stopImmediatePropagation();
			loadInfo(previousIDs.pop(), true);
			if (previousIDs.length == 0) backButton.removeEventListener(MouseEvent.CLICK, back);
		}
		
		// sets previous panel
		override public function set previousPanel(value:int):void 
		{
			super.previousPanel = value;
			if (value == -1) {
				currentID = null;
				previousIDs.length = 0;
				backButton.removeEventListener(MouseEvent.CLICK, back);
			}
		}
		
		// insert current location at beginning of queue
		private function insertBefore(e:MouseEvent = null):void 
		{
			dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Destination Set", currentData.name + TOAST_INSERT_BEFORE));
			dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "insertBefore", id: currentID } ));
		}
		
		// insert current location at end of queue
		private function insertAfter(e:MouseEvent = null):void 
		{
			dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Destination Added", currentData.name + TOAST_INSERT_AFTER));
			dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "insertAfter", id: currentID } ));
		}
		
	}

}