package ui.panel
{
	import flash.events.Event;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import map.Destination;
	import map.MapData;
	import ui.RouteEvent;
	import ui.Spritesheet;
	import ui.RouteSlot;
	import ui.ToastEvent;
	import ui.RouteHUD;

	/**
	 * RoutePlannerPanel
	 * 
	 * @author Charlie Chao
	 */
	public class RoutePlannerPanel extends Panel 
	{
		/**
		 * Constants
		 */
		public static const WIDTH:int = 260;
		public static const CONTENT_WIDTH:int = WIDTH - PADDING * 2;
		public static const HELP:String = "Tap on points of interest and add them to queue. Then re-arrange them here to plan your route.<br><br>";
		
		/**
		 * Attributes
		 */
		private var routeList:Vector.<Destination>;
		private var distance:Number;
		
		/**
		 * Components
		 */
		public var routeSlots:Vector.<RouteSlot>;
		private var infoText:TextField;
		
		public function RoutePlannerPanel(routeList:Vector.<Destination>) 
		{
			super(WIDTH, Main.CATEGORY_ROUTE_PLANNER, "side_button_route_planner", "route_planner");
			title.text = "Route Planner";
			this.routeList = routeList;
			
			// setup distance textfield
			infoText = new TextField();
			infoText.x = infoText.y = PADDING;
			infoText.y += CONTENT_OFFSET;
			infoText.defaultTextFormat = CommonStyle.INFO_FORMAT_TEXT;
			infoText.autoSize = TextFieldAutoSize.LEFT;
			infoText.selectable = false;
			infoText.width = CONTENT_WIDTH;
			infoText.multiline = true;
			infoText.wordWrap = true;
			infoText.htmlText = HELP + "Getting route list...";
			
			// adjust scrollable content pane position
			content.y += infoText.height + PADDING * 2;
			
			addChild(content);
			addChild(infoText);
		}
		
		// added to screen handler
		override public function added(e:Event = null):void 
		{
			super.added(e);
			addEventListener(PanelEvent.PANEL_CHANGE, slotChange);
			update();
			dispatchEvent(new RouteEvent(RouteEvent.CHANGE, null));
		}
		
		// removes the panel from screen and removes all the listeners
		override protected function dispose():void 
		{
			dispatchEvent(new RouteEvent(RouteEvent.CHANGE, null));
			removeEventListener(PanelEvent.PANEL_CHANGE, slotChange);
			Main.selectedDestination = -1;
			super.dispose();
		}
		
		// slot change handler
		private function slotChange(e:PanelEvent):void 
		{
			var target:RouteSlot = RouteSlot(e.target); 
			if (e.params != null) {
				if (e.params == "goUp") {
					if (target.id != 0) {
						// move up
						dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "swap", id: target.id - 1, id2: target.id } ));
					}
				} else if (e.params == "goDown") {
					if (target.id != routeList.length - 1) {
						// move down
						dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "swap", id: target.id + 1, id2: target.id } ));
					}
				} else if (e.params == "select") {
					// select
					Main.selectedDestination = RouteSlot(e.target).id;
					update();
					dispatchEvent(new ToastEvent(ToastEvent.SHOW, "Adding Intersection", "Tapping the map will insert an intersection before the selected destination. Deselect the destination to stop adding."));
				} else {
					Main.selectedDestination = -1;
					update();
				}
			} else {
				// remove
				Main.selectedDestination = -1;
				dispatchEvent(new RouteEvent(RouteEvent.CHANGE, { action: "remove", id: target.id } ));
			}
		}
		
		// updates panel
		override public function update():void
		{
			// remove previous slots
			if (routeSlots != null) {
				routeSlots.length = 0;
				content.clear();
			}
			
			// populate new slots
			routeSlots = new Vector.<RouteSlot>;
			distance = 0; 
			for (var i:int = 0; i < routeList.length; i++) {
				routeSlots.push(new RouteSlot(i, routeList[i].name, routeList[i].distance + distance, WIDTH, uiSpriteSheet.getSprite(routeList[i].icon + "_small")));
				content.add(routeSlots[i]); 
				distance += routeList[i].distance;
			} 
			infoText.htmlText = HELP + "<b>Total Distance:</b> " + Math.round(RouteHUD.CONVERSION * distance) + " m";
		}
		
	}

}